{{
  ----------------------------------------------
  Propeller Based Webserver
  ----------------------------------------------
}}

CON

  _clkmode = xtal1+pll16x
  '_xinfreq = 6_000_000
  '_xinfreq = 6_553_600
  _xinfreq = 5_000_000

  RXLEN = 1024
  TXLEN = 7680
  
OBJ

  web     : "Brilldea_W5100_indirect_Driver_Basepin_Ver001"
  'text    : "parallel_lcd_pexp_mod"
  str     : "util_strings"
  sdfat   : "fsrw"
  time    : "Timing"

VAR

  byte bSend
  long webStack[64]
  long size
  long cardsize
  byte position 
  byte sendwebBuffer[TXLEN]
  byte receivewebBuffer[RXLEN]
  byte temp
  byte fname[13]
  byte fending[4] 
  long type
  byte reqstr[32]               ' request string
  long MACcheck, IPcheck, SUBNETcheck 

DAT
  mac_addr      byte    $10, $00, $00, $00, $00, $01
  ip_addr       byte    192, 168, 178, 4                   ' device's ip address (adjust to your network configuration)
  ip_subnet     byte    255, 255, 255, 0                   ' network subnet
  ip_gateway    byte    192, 168, 178, 1                   ' network gateway (adjust to your network configuration)
  ip_dns        byte    10, 10, 10, 254                    ' network dns
  ip_destination byte   0, 0, 0, 0
  dhcp_or_static byte   0

PUB start | temp0

  temp0:=sdfat.mount_explicit(12,11,10,9)
  web.startINDIRECT(16,24,8,25,0,27,26,true)
  web.InitAddresses(true,@mac_addr,@ip_gateway,@ip_subnet,@ip_addr)
  '''web.start(11, 10, 9, 8, -1, -1, @mac_addr, @ip_addr)
  web.SetSocketMemory(true,0,web#_1KB,web#_8KB)
  {text.start                          ' This is just for debugging purposes
  text.out($00)                        
  text.str(string("Server ready"))      
  text.pos(0,2)
  text.str(string("MAC: "))
  MACcheck:=web.ReadMACAddress
  text.hex(byte[MACcheck],2)
  text.hex(byte[MACcheck+1],2)
  text.hex(byte[MACcheck+2],2)
  text.hex(byte[MACcheck+3],2)
  text.hex(byte[MACcheck+4],2)
  text.hex(byte[MACcheck+5],2)
  text.pos(0,3)
  text.str(string("IP:  "))
  IPcheck:=web.ReadIPAddress
  text.dec(byte[IPcheck])
  text.out(".")
  text.dec(byte[IPcheck+1])
  text.out(".")
  text.dec(byte[IPcheck+2])
  text.out(".")
  text.dec(byte[IPcheck+3])
  text.pos(0,4)
  text.str(string("SUB: "))
  SUBNETcheck:=web.ReadSubnetMask
  text.dec(byte[SUBNETcheck])
  text.out(".")
  text.dec(byte[SUBNETcheck+1])
  text.out(".")
  text.dec(byte[SUBNETcheck+2])
  text.out(".")
  text.dec(byte[SUBNETcheck+3])}

  if temp0 == 0
  '''ifnot \sdfat.mount(12)
    cardsize:=(sdfat.getclustercount/1024)
    cardsize:=cardsize*sdfat.getclustersize/1024
    cognew(webServer, @webStack)
  {else
    text.out($00)
    text.str(string("SD Card error"))} 

PRI webServer | temp0
  ' web server (runs in a seperate cog)
  \web.SocketOpen(0, web#_TCPPROTO, 80, 80, @ip_destination)
  \web.SocketTCPlisten(0)
  repeat
    if web.SocketTCPestablished(0)==true
        webServerThread
        \web.SocketTCPdisconnect(0)
        time.pause1ms(200)
        \web.SocketClose(0)
        \web.SocketOpen(0, web#_TCPPROTO, 80, 80, @ip_destination)
        \web.SocketTCPlisten(0)
    else
      next  

DAT

http200       byte      "HTTP/1.0 200 OK",13,10,0
http404       byte      "HTTP/1.0 404 Not Found",13,10,13,10,0
http501       byte      "HTTP/1.0 501 Not Implemented",13,10,13,10,0
http503       byte      "HTTP/1.0 503 Service Unavailable",13,10,13,10,0 
filelist      byte      "<html>Liste der Dateien auf dem Webserver:<br>------------------------------------<br><br>",0
crlf          byte      13,10,0

PRI webServerThread | ptr, row, c, h, j, n, m, i, temp0
  ' handles a single client connection
  m:=0
  'text.out($00)
  bytefill(@receivewebBuffer,0,RXLEN)
  time.pause1ms(20)
  web.rxTCP(0,@receivewebBuffer)       
  
  'text.str(string("Received:"))
  'text.out(c)
  
  bytemove(@reqstr, @receivewebBuffer, 32)     ' copy the header to a temporary request string for later processing

  if str.indexOf(@reqstr, string("GET")) <> -1
    'text.pos(0,1)
    'text.str(string("Received GET"))     
  elseif str.indexOf(@reqstr, string("POST")) <> -1
    'text.pos(0,1)
    'text.str(string("Received POST"))  
  elseif str.indexOf(@reqstr, string("HEAD")) <> -1
    'text.pos(0,1)
    'text.str(string("Received HEAD"))
    m:=1
  else
    web.str(0, @http501)
    return -1                   ' unrecognized method / not implemented; send 501 error message

  j:=0
  h:=0
  i:=0
  temp:=0

  position:=str.indexOf(@reqstr, string(" /"))+2
  repeat until j==12
    fname[j]:=reqstr[position]              ' reading the filename (it has to be in the 8.3 format or shorter)
    if fname[j]==32
     temp:=j
     fname[j+1]:=0                          ' let the filename be a zero terminated string (not necessary for fsrw, but avoiding problems when using strcomp)
     quit
    position++
    j++ 

  
  if fname[0]==32                           ' if there is no filename in the request, send a list of files which are stored on the sd card
    'text.pos(0,2)
    'text.str(string("show filelist"))
    sdfat.opendir
    web.str(0,@http200)
    web.str(0,string("Server: PropServer"))  ' but first we are going to send the header
    web.str(0,@crlf)                         ' which includes some information about the server and file
    web.str(0,string("Content-Type: "))
    web.str(0,string("text/html"))
    web.str(0,@crlf)
    web.str(0,string("Connection: close"))
    web.str(0,@crlf)
    web.str(0,@crlf)
    web.str(0,@filelist)
    repeat until sdfat.nextfile(@fname)==-1
      temp++                                ' !!! Exception: I'm using the same variable for two different purposes (file length [see above] <-> number of files [here]) !!!
      web.str(0,string("<a href="))
      web.str(0,string(34))
      web.str(0,string("/"))
      web.str(0,@fname)
      web.str(0,string(34))
      web.str(0,string(">"))
      web.str(0,@fname)
      web.str(0,string("</a>")) 
      web.str(0,string("<br>"))
    'web.str(string("<br>"))
    web.str(0,string("------------------------------------"))
    web.str(0,string("<br>"))
    web.str(0,string("number of files: "))
    web.dec(0,temp)
    web.str(0,string("<br>"))
    web.str(0,string("<br>"))
    'web.str(0,string("Dateisystem: "))
    {if sdfat.getfilesystem == 1
      web.str(0,string("FAT 16"))
    elseif sdfat.getfilesystem == 2
      web.str(0,string("FAT 32"))
    else
      web.str(0,string("FEHLER: unbekanntes Dateisystem"))}
    web.str(0,string("<br>"))
    web.str(0,string("storage capacity: "))
    web.dec(0,cardsize)
    web.str(0,string(" MB"))
    web.str(0,string("</html>"))
    return 0

  
  if temp==0                                ' get the file extension...
    fending[0]:=fname[9]                    ' if the filename matches the 8.3 format, it's the last three entries of fname
    fending[1]:=fname[10]
    fending[2]:=fname[11]
  else 
    fending[0]:=fname[temp-3]               ' otherwise temp marks the space after the filename
    fending[1]:=fname[temp-2]
    fending[2]:=fname[temp-1]
  fending[3]:=0                             ' zero terminated string in order to use strcomp
  
  if strcomp(@fending, string("htm"))       ' ... and decide what MIME type it is
    type := string("text/html")             ' of course there are a lot more, but these
  elseif strcomp(@fending, string("avi"))   ' I think are the most common ones
    type := string("video/x-msvideo")       ' or at least the ones I needed ;-)
  elseif strcomp(@fending, string("mpg"))
    type := string("video/mpeg")
  elseif strcomp(@fending, string("jpg"))
    type := string("image/jpeg")
  elseif strcomp(@fending, string("gif"))
    type := string("image/gif")
  elseif strcomp(@fending, string("pdf"))
    type := string("application/pdf")
  elseif strcomp(@fending, string("zip"))
    type := string("application/zip")
  elseif strcomp(@fending, string("wav"))
    type := string("audio/x-wav")
  elseif strcomp(@fending, string("mp3"))
    type := string("audio/mpeg")
  elseif strcomp(@fending, string("m4a"))
    type := string("audio/x-m4a")
  else
    type := string(" ")

  'text.pos(0,3)
  'text.str(string("requested file:"))
  'text.pos(0,4)
  'text.str(@fname)
  if sdfat.popen(@fname,"r") <> 0            ' Try to open the requested file
    web.str(0,@http404)                      ' if it is not found on the sd card send back the 404 error message
  else                                       
    size:=sdfat.get_filesize
    web.str(0,@http200)                      ' file has been found and will be transmitted,
    web.str(0,string("Server: PropServer"))  ' but first we are going to send the header
    web.str(0,@crlf)                         ' which includes some information about the server and file                                
    web.str(0,string("Content-Length: "))
    web.dec(0,size)
    web.str(0,@crlf)
    web.str(0,string("Content-Type: "))
    web.str(0,type)
    web.str(0,@crlf)
    web.str(0,string("Connection: close"))
    web.str(0,@crlf)
    web.str(0,@crlf)                         ' This is the end of the Header 
    if m==1                                  ' a HEAD request would stop here
      return 0
    else
      repeat while web.SocketTCPestablished(0)==true
        h:=sdfat.pread(@sendwebBuffer,txlen)  ' now we are reading the first BufferSize (4096 default) bytes
        if h=<0                               ' if there are no bytes to read then exit the loop                          
          quit
        else
          web.txTCP(0,@sendwebBuffer,h)            ' send the h bytes read      
          
  return 0